SystemOrganization addCategory: #'System-Tools'!

Object subclass: #ToolRegistry
	instanceVariableNames: 'tools inspectorsMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tools'!

!ToolRegistry commentStamp: 'BernardoContreras 1/18/2012 21:44' prior: 0!
I am serving as a system-wide hub for various tools.
Instead of using direct class names for tools, which is not always nice because can introduce inter-package
dependencies, i provide a uniform way for accessing tools, via Smalltalk global:

Smalltalk tools someToolName,

where 'someToolName' is a name of the tool , under which some tool are registered.

Tools are adding themselves to registry by implementing #registerToolsOn: message at class side, for example:

MyClass>>registerToolsOn: registry
   registry register: self as: #myTool

Registers a MyClass as a tool under name #myTool, and can be accessed via:

Smalltalk tools myTool 

and avoid putting 'MyClass' reference into code.

You can add  ' self registerToolsOn: Smalltalk tools ' in class #initialize,
so your new tool(s) can be registered during package loading.

To reset tool registry to defaults , use: 
Smalltalk resetTools.

Also registry storing a map of inspector types. 
This is to replace an old scheme which using #inspectorClass , and introducing dependencies.
Now, an inspecting class dont needs to have any knowledge about its specialized inspector.
Instead, a specialized inspector could tell registry that it is available for inspecting instances of given class (see #registerInspector:for:  senders).
This allows to avoid dependency from instance class to its inspector class or using extension methods.

WARNING: The point of the tools registry is NOT to be a facade full of protocols but to be a registration for tools that get loaded. So, the tools registry should NOT hold specific tool's protocol.  the tool's client should invoke the correct methods of the tool.

----------- 
The category 'menu' is for backward compatibility with ToolSet. We should remove it.

!

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 3/6/2011 18:03'!
browseDirectToolReferences
	"Smalltalk tools browseDirectToolReferences"
	"Browse all direct references of all of my registered tools.
	Useful for converting calls like:

		MyTool foo
	into:
		Smalltalk tools myTool foo 	"
	
	| bindings sn |
	
	bindings := tools values select: [:each | each value isBehavior ]
		thenCollect: [:each | each value binding ].
	bindings := bindings copyWith: (Smalltalk bindingOf: #ToolSet).
	
	sn := SystemNavigation default.	

	^ sn
		browseMessageList: (
			sn allMethodsSelect: [:m |
				(m literals allButLast select: [:lit| lit isVariableBinding ]) includesAnyOf: bindings ] )
		name: 'All direct references to tools to fix'! !

!ToolRegistry methodsFor: 'misc' stamp: 'MarcusDenker 5/27/2011 14:24'!
codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt
	^(self hasToolNamed: #codeCompletion)
	ifTrue: [ self codeCompletion codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt ]
	ifFalse: [ aBlock value ]! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:02'!
debugError: anError
	"Handle an otherwise unhandled error"
	^Processor activeProcess
		debug: anError signalerContext
		title: anError description! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'CamilloBruni 2/9/2012 12:28'!
debugSyntaxError: aSynataxError
	"Handle a syntax error.
	
	Note, if there is no #syntaxError registered, it will be silently ignored"
	
	self using: #syntaxError do: [:tool | 
			tool open: (tool syntaxError: aSynataxError)]! !

!ToolRegistry methodsFor: 'handling DNU' stamp: 'IgorStasenko 2/19/2011 03:15'!
doesNotUnderstand: aMessage
	"Return a tool identified by a message selector.
	We need to handle special case, when no debugger registered.
	"
	| tool |
	tool := tools at: aMessage selector ifAbsent: nil.
	tool ifNotNil: [ ^ tool value ].

	^ (aMessage selector == #debugger) 
		ifTrue: [ 
			Smalltalk 
				logError: 'No debugger installed to handle debug request.' 
				inContext: thisContext.
		
			nil. "Smalltalk snapshot: false andQuit: true."
		]
		ifFalse: [ super doesNotUnderstand: aMessage ]
! !

!ToolRegistry methodsFor: 'testing' stamp: 'IgorStasenko 2/19/2011 03:18'!
hasToolNamed: aToolName
	^ tools includesKey: aToolName ! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 10/29/2008 02:09'!
informNoDebugger
	(self includes: #uiManager) 
		ifTrue: [
			self uiManager inform: 'Cannot open debugger' ]
		ifFalse: [
			"If we throw an exception here, this can lead to recursion.
			Any idea what else we can do here?"
			Processor activeProcess terminate ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 8/12/2011 16:37'!
initDefaultToolSet
	
	"Query the system for registering tools"
	"we cannot afford to show warnings because not all tools e.g Debugger, Inspector are registered yet"
	
	[	
	Object withAllSubclassesDo: [:ea | 
		
	"evaluate only for classes which directly implement this message,
	not via subclassing"
		(ea isMeta  and: [
			ea includesSelector: #registerToolsOn: ]) ifTrue: [
			 ea theNonMetaClass registerToolsOn: self  ]].
	] on: Warning do: [:ex | ex resume ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 2/20/2011 14:58'!
initialize

	"symbols used as keys "
	tools := IdentityDictionary new.  
	inspectorsMap := IdentityDictionary new.
	
	self initDefaultToolSet! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:03'!
inspect: anObject
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass "
	
	| class |
	
	class := (self inspectorClassOf: anObject)
		ifNil: [ tools at: #inspector ifAbsent: [ self error: 'no inspector found' ]].
	
	class ifNotNil: [ ^ class openOn: anObject ]
	
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'GaryChambers 12/12/2011 11:20'!
inspect: anObject label: aString
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass 
	"
	| class |
	
	class := self inspectorClassOf: anObject. 
	class ifNil: [ class := tools at: #inspector ifAbsent: []].
	
	class ifNotNil: [ ^ class openOn: anObject withEvalPane: true withLabel: aString ]
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:02'!
inspectorClassOf: anObject
	"Answer the inspector class for the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
		
	anObject class withAllSuperclassesDo: [:cls | 
		inspectorsMap at: cls name ifPresent: [:inspectorName |
			^ Smalltalk classNamed: inspectorName
		]].

	^ nil
	
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:26'!
mainMenuItems
	"Answer the most important menu items available for this tool set"
	^#(
		('System Browser' 			#openClassBrowser)
		('Workspace'				#openWorkspace)
		('Test Runner'				#openTestRunner)
		('Monticello Browser'		#openMonticelloBrowser)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 15:32'!
menuItems
	"Answer the menu items available for this tool set"

	^#(
		('System Browser' 			#openClassBrowser)
		('Message Names'			#openMessageNames)
		-
		('Workspace'				#openWorkspace)
		('Transcript' 				#openTranscript)
		('File Browser'				#openFileList)
		-
		('Test Runner'				#openTestRunner)
		('Process Browser' 			#openProcessBrowser)
		-
		('Monticello Browser'		#openMonticelloBrowser)
		"('Monticello Configurations' #openMonticelloConfigurations)"
		('Recover lost changes...'	#openRecentChangesLog)
		"('Simple Change Sorter'		#openChangeSorter)"
		('Change Sorter'				#openDualChangeSorter)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:57'!
openChangeSorter
	^ self changeSorter open
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:55'!
openClassBrowser
	^ self browser open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:42'!
openDualChangeSorter
	self dualChangeSorter open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:38'!
openFileList
	
	self fileList open! !

!ToolRegistry methodsFor: 'menu' stamp: 'BernardoContreras 8/14/2011 02:23'!
openFinder
	^ self finder open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:42'!
openMessageNames
	"Bring a MessageNames tool to the front"
	
	self using: #messageNames do: [:tool | tool openMessageNames ]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'sd 6/5/2011 19:05'!
openMonticelloBrowser
	
	self monticelloBrowser open
	! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:43'!
openProcessBrowser
	self using: #processBrowser do: [:tool |
		tool open
		]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:39'!
openRecentChangesLog
	self changeList browseRecentLog! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:46'!
openSelectorBrowser
	self using: #selectorBrowser do: [:tool |
		tool new morphicWindow openInWorld
	]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 04:02'!
openTestRunner
	self using: #testRunner do: [:tool | tool open ]! !

!ToolRegistry methodsFor: 'menu' stamp: 'FernandoOlivero 5/9/2011 09:38'!
openTranscript

	^ UIManager default openTranscript! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:58'!
openWorkspace
	^ self workspace open! !

!ToolRegistry methodsFor: 'registration' stamp: 'BenjaminVanRyseghem 2/22/2012 14:53'!
register: anObject as: aNameSelector
	"Register a tool class under given name"
	tools at: aNameSelector put: anObject.
	WorldState defaultWorldMenu! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 14:58'!
registerInspector: anInspectorClass for: anInstanceClass

	"Register inspector class for given instance class" 

	inspectorsMap at: anInstanceClass name put: anInspectorClass name
! !

!ToolRegistry methodsFor: 'registration' stamp: 'IgorStasenko 10/29/2008 01:31'!
remove: aName
	"remove tool with given name"
	tools removeKey: aName ifAbsent: []! !

!ToolRegistry methodsFor: 'tools' stamp: 'IgorStasenko 2/20/2011 14:36'!
transcript
	"No need to register transcript. But for uniformity,
	implement this method, so transcript can be accessed via
	Smalltalk tools transcript , as any other tool "
	^ Transcript! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 2/19/2011 04:03'!
using: aToolName do: aBlock
	"Evaluate a block for tool found under corresponding name or silently ignore if its not found or nil"
	| tool |
	
	tool := tools at: aToolName ifAbsent: [ ^ self ].
	tool ifNotNil: [ ^ aBlock value: tool value ]! !
